//---------------------------------------------------------------------------
#ifndef comthreadH
#define comthreadH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
namespace ComThread
{       
//---------------------------------------------------------------------------
typedef void __fastcall (__closure *TComThreadEvent)( System::TObject* Sender, AnsiString &buffer);
//---------------------------------------------------------------------------
class TComport;  // forward declaration -- hides implementation of threading
//---------------------------------------------------------------------------
// component class  for comport -- should be on the component palette
//---------------------------------------------------------------------------
class TComThread : public TComponent
{
  // DATA
public:
	bool ok;
	AnsiString errMsg;

	enum TParity { none=NOPARITY, odd=ODDPARITY, even=EVENPARITY, mark=MARKPARITY, space=SPACEPARITY };
  enum TStopBits { s1=ONESTOPBIT, s1_5=ONE5STOPBITS, s2=TWOSTOPBITS };

private:
	TComport *com;  // holds the TThread derived class
	HANDLE hCom;    // a handle to the comport
  TOverlapped *rOLap, *wOLap;

	// published data
	unsigned long FBufSize;
	unsigned long FBaud;
	unsigned int FDataBits;
	TParity FParity;
	TStopBits FStopBits;
	unsigned int FPort;
  unsigned long FPacketSize;
  unsigned long FTimeout;
	TComThreadEvent FOnPacketReceived;

	// METHODS
private:
  void InitPort();  // open the serial port

protected:
	virtual void __fastcall SetBufSize( unsigned long b);
	virtual void __fastcall SetBaud( unsigned long b);
	virtual void __fastcall SetDataBits( unsigned int d);
	virtual void __fastcall SetParity( TParity p);
	virtual void __fastcall SetStopBits( TStopBits s);
	virtual void __fastcall SetPort( unsigned int p);
  virtual void __fastcall SetPacketSize( unsigned long p);
  virtual void __fastcall SetTimeout( unsigned long t);

public:
	__fastcall TComThread(TComponent* Owner);		// constructor
  __fastcall ~TComThread(){ ComClose(); };
	void __fastcall ComInit();									// start up the comport
	void __fastcall ComClose();									// shut it down
	void __fastcall Write( AnsiString &s);			// output a string

	// properties
__published:
	__property unsigned long BufSize = {read=FBufSize, write=SetBufSize, default=128};
	__property unsigned long Baud = {read=FBaud, write=SetBaud, default=19200};
	__property unsigned int DataBits = {read=FDataBits, write=SetDataBits, default=8};
	__property TParity Parity = {read=FParity, write=SetParity, default=none};
	__property TStopBits StopBits = {read=FStopBits, write=SetStopBits, default=1};
	__property unsigned int Port = {read=FPort, write=SetPort, default=1};
  __property unsigned long PacketSize = {read=FPacketSize, write=SetPacketSize, default=80};
  __property unsigned long Timeout = {read=FTimeout, write=SetTimeout, default=100};
	__property TComThreadEvent OnPacketReceived = {read=FOnPacketReceived, write=FOnPacketReceived};
};
//---------------------------------------------------------------------------
} /* namespace ComThread */
#if !defined(NO_IMPLICIT_NAMESPACE_USE)
using namespace ComThread;
#endif
//---------------------------------------------------------------------------
#endif
